/*
 * Copyright 2012 NGDATA nv
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.yihu.ehr.sep;

import com.ngdata.sep.EventListener;
import com.ngdata.sep.PayloadExtractor;
import com.ngdata.sep.SepEvent;
import com.ngdata.sep.SepModel;
import com.ngdata.sep.impl.BasePayloadExtractor;
import com.ngdata.sep.impl.SepConsumer;
import com.ngdata.sep.impl.SepModelImpl;
import com.ngdata.sep.util.zookeeper.ZkUtil;
import com.ngdata.sep.util.zookeeper.ZooKeeperItf;
import org.apache.commons.codec.StringEncoder;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.Cell;
import org.apache.hadoop.hbase.CellUtil;
import org.apache.hadoop.hbase.HBaseConfiguration;
import org.apache.hadoop.hbase.util.Bytes;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.Producer;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.core.KafkaTemplate;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

/**
 * A simple consumer that just logs the events.
 */
public class LoggingConsumer {


    private static String localhost = "192.168.131.109";        //localhost
    private static String zookeeperHost = "master";             //zookeeper
    private static String subscriptionName = "logger";
    private static byte[] table = Bytes.toBytes("sep-user-demo");       //hbase
    private static byte[]  columnFamily = Bytes.toBytes("");        //
    private static byte[]  columnQualifier = Bytes.toBytes("payload");  //

    private static String index = "medical_service_index";
    private static String type = "medical_service";







    public static void main(String[] args) throws Exception {
        Logger log = LoggerFactory.getLogger(LoggingConsumer.class);
        log.debug("in sep Events ");
        Configuration hbaseConf = HBaseConfiguration.create();
        hbaseConf.setBoolean("hbase.replication", true);

        ZooKeeperItf zk = ZkUtil.connect(zookeeperHost, 20000);
        SepModel sepModel = new SepModelImpl(zk, hbaseConf);
        if (!sepModel.hasSubscription(subscriptionName)) {
            sepModel.addSubscriptionSilent(subscriptionName);
        }

        PayloadExtractor payloadExtractor = new BasePayloadExtractor(table, columnFamily,columnQualifier);
        EventListener eventLogger = new EventLogger();
        SepConsumer sepConsumer = new SepConsumer(subscriptionName, 5000, eventLogger, 1, localhost, zk, hbaseConf,payloadExtractor);
        sepConsumer.start();

        System.out.println("Started");

        while (true) {
            Thread.sleep(Long.MAX_VALUE);
        }
    }

    private static Producer createProducer() {
        Properties properties = new Properties();
        properties.put("zookeeper.connect", "192.168.131.234:2181");//
        properties.put("serializer.class", StringEncoder.class.getName());
        properties.put("bootstrap.servers", "192.168.131.234:9092");//
        properties.put("key.serializer", "org.apache.kafka.common.serialization.StringSerializer");
        properties.put("value.serializer", "org.apache.kafka.common.serialization.StringSerializer");
        Producer<String, String> producer = new KafkaProducer<String, String>(properties);
        return producer;
    }

    private static void sendMessage(Producer producer ,String topic,String message){
//            for (int i = 0; i < 100; i++) {
//                ProducerRecord producerRecord = new ProducerRecord<String, String>(topic, Integer.toString(i), Integer.toString(i));
//                producer.send(producerRecord);
//            }

        producer.send(new ProducerRecord<Integer, String>(topic, "message: " + message));
        producer.close();
    }

    private static class EventLogger implements EventListener {
        private static String topic = "sep-hbase-data";
        @Autowired
        private KafkaTemplate<String,String> kafkaTemplate;
        Logger log = LoggerFactory.getLogger(LoggingConsumer.class);

        @Override
        public void processEvents(List<SepEvent> sepEvents) {
            log.info(" sep Events get listener");

            Producer producer =  createProducer();

            for (SepEvent sepEvent : sepEvents) {
                System.out.println("Received event:");
                System.out.println("  table = " + Bytes.toString(sepEvent.getTable()));
                System.out.println("  row = " + Bytes.toString(sepEvent.getRow()));
                log.debug("  table = " + Bytes.toString(sepEvent.getTable()));
                log.debug("  row = " + Bytes.toString(sepEvent.getRow()));
                Map<String, Object> json = new HashMap<String, Object>();
                for (Cell cell : sepEvent.getKeyValues()) {
                    String key = Bytes.toString(CellUtil.cloneQualifier(cell));
                    String value = Bytes.toString(CellUtil.cloneValue(cell));
                    log.debug("hbase  cloumn = " + key);
                    log.debug("hbase  value = " + value);
                    System.out.println("  key = " + key);
                    System.out.println("  value = " + value);
                    sendMessage(producer,topic,key + ":" + value);
//                    kafkaTemplate.send(topic,"你","好");
                    json.put(key, value);
                }
            }
            log.info("sep Events end");
        }


    }
}
